/* NetStats - IRC Statistical Services
** Copyright (c) 1999 Adam Rutter, Justin Hammond
** http://codeworks.kamserve.com
*
** Based from GeoStats 1.1.0 by Johnathan George net@lite.net
*
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
** See the file called 'LICENSE' for more details.
*/

#include "stats.h"

char s_StatServ[MAXNICK] = "StatServ";
char s_Spam[8] = "cm-i";
char s_Debug[MAXNICK] = "Stats_Debug";
const char version[] = "NeoStats-1.1.2";
const char version_date[] = __DATE__;
const char version_time[] = __TIME__;
const char protocol_version[] = "2106";

static void start();

int main()
{
	int forked = 0;
	FILE *fp;
	me.onchan = 0;
	log("Statistics Started (%s).", version);
	me.t_start = time(NULL);
	me.want_privmsg = 0;
	me.enable_spam = 0;
	me.coder_debug=0;
	me.noticelag=0;

	ConfLoad();
	LoadStats();
	LoadTLD();

#ifndef DEBUG
	forked=fork();
#endif
	if (forked) {
		fp = fopen("stats.pid", "w");
		fprintf(fp, "%i", forked);
		fclose(fp);
		return 0;
	}

	start();

	return 1;
}

void start()
{
	TimerReset();
	me.users = me.opers = me.servers = me.collisions = 0;
	init_server_hash();
	init_user_hash();
	init_tld();

	log("Connecting to %s:%d", me.uplink, me.port);
	if (servsock > 0)
		close(servsock);

	servsock = ConnectTo(me.uplink, me.port);

	if (servsock <= 0) {
		log("Unable to connect to %s", me.uplink);
	} else {
		login();
		read_loop();
	}
	log("Reconnecting to the server in %d seconds", me.r_time);
	sleep(me.r_time);
	start();
}

void login()
{
	sts("PASS %s", me.pass);
	sts("SERVER %s 1 :%s", me.name,me.infoline);
}

void init_statserv()
{
	sts("NICK %s 1 %d %s %s %s 0 :/msg %s \2HELP\2", s_StatServ, time(NULL),
		StatServ.user, StatServ.host, me.name, s_StatServ);
	AddUser(s_StatServ, StatServ.user, StatServ.host, me.name);
	sts(":%s JOIN %s",s_StatServ ,me.chan);
	sts(":%s MODE %s +o %s",me.name,me.chan,s_StatServ);
	sts(":%s MODE %s +a %s",s_StatServ,me.chan,s_StatServ);
}

void init_spam()
{
	if (!me.enable_spam)
		return;

	sts("NICK %s 1 1 please %s %s 0 :Spam Me!", s_Spam,
		me.name, me.name);
	AddUser(s_Spam, "please", me.name, me.name);
	log("Spam Name: %s -=(%s)=-",s_Spam, me.name);
}

void *smalloc(long size)
{
	void *buf;

	if (!size) {
		log("smalloc(): illegal attempt to allocate 0 bytes!");
		size = 1;
	}
	buf = malloc(size);
	if (!buf) {
		log("smalloc(): out of memory.");
		exit(0);
	}
	return buf;
}

char *sstrdup(const char *s)
{
	char *t = strdup(s);
	if (!t) {
		log("sstrdup(): out of memory.");
		exit(0);
	}
	return t;
}

unsigned long HASH(const unsigned char *name, int size_of_table)
{
	unsigned long h = 0, g;

	while (*name) {
		h = (h << 4) + *name++;
			if ((g = h & 0xF0000000))
				h ^= g >> 24;
			h &= ~g;
	}
	return h % size_of_table;
}

char *strlower(char *s)
{
	char *t = s;
	while ((*t++ = tolower(*t)))
		;
	return s;
}
