/* NetStats - IRC Statistical Services
** Copyright (c) 1999 Adam Rutter, Justin Hammond
** http://codeworks.kamserve.com
*
** Based from GeoStats 1.1.0 by Johnathan George net@lite.net
*
** NetStats CVS Identification
** $Id$
*/

#ifndef STATS_H
#define STATS_H

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <errno.h>
#include <netdb.h>
#include <netinet/in.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>
#include <sys/time.h>
#include <ctype.h>
#include <sys/stat.h>
#include "m_stats.h"

#undef  DEBUG
#define BUFSIZE			512
#define CONFIG_NAME		"stats.cfg"
#define MAXHOST			63
#define MAXPASS			32
#define MAXNICK			32
#define MAXUSER			10
#define S_TABLE_SIZE	97
#define U_TABLE_SIZE	1999
#define D_TABLE_SIZE	1999
#define bzero(x, y)		memset(x, '\0', y);

int servsock;
extern char s_StatServ[MAXNICK], s_Spam[8], s_Debug[MAXNICK];
extern const char version[];

typedef struct server_ Server;
typedef struct user_ User;
typedef struct stats_ Stats;
typedef struct tld_ TLD;
typedef struct myuser_ MyUser;

struct me {
	char name[MAXHOST];
	int port;
	int r_time;
	int lag_time;
	char uplink[MAXHOST];
	char pass[MAXPASS];
	char services_name[MAXHOST];
        char infoline[MAXPASS];
        char netname[MAXPASS];
	time_t t_start;
	unsigned int enable_spam : 1;
	unsigned int want_privmsg : 1;
	unsigned int send_extreme_lag_notices : 1;
	unsigned int onlyopers : 1;
	unsigned int collisions;
	unsigned int enable_proxy : 1;
	unsigned int coder_debug : 1;
	unsigned int noticelag : 1;
	int action;
	char *message;
	char *chan;
	unsigned int onchan : 1;
	int opers;
	int maxopers;
	long users;
	long maxusers;
	int servers;
	int maxservers;
	time_t t_maxopers;
	time_t t_maxusers;
	time_t t_maxservers;
	unsigned int away;
	Server *s;
	int requests;
} me;

#define ACTION_WARN			0x00001
#define ACTION_KILL			0x00002
#define ACTION_WARN_U		0x00004
#define ACTION_BAN			0x00008
#define ACTION_USE_SERVICES 0x00010

struct StatServ {
	char user[MAXUSER];
	char host[MAXHOST];
} StatServ;

struct IcqServ {
	char user[MAXUSER];
	char host[MAXHOST];
	long uin;
	long passwd;
} IcqServ;

struct server_ {
	Server *next, *prev;
	char name[MAXHOST];
	int hops;
	long hash;
	Stats *stats;
	unsigned int users;
	int opers;
	time_t connected_since;
	time_t last_announce;
	int ping;
	int lowest_ping;
	int highest_ping;
	time_t t_lowest_ping;
	time_t t_highest_ping;
	Server *uplink;
};

struct user_ {
	User *next, *prev;
	char nick[MAXNICK];
	char *hostname;
	char *username;
	Server *server;
	TLD *tld;
	MyUser *myuser;
	int flood;
	time_t t_flood;
	long hash;
	unsigned int is_oper : 1;
	unsigned int is_away : 1;
	unsigned int is_coder : 1;
	unsigned int is_netmin : 1;
	unsigned int is_tecmin : 1;
	unsigned int is_srvmin : 1;
	unsigned int is_comin : 1;
	unsigned int is_serv : 1;
	unsigned int is_invis : 1;
	unsigned int is_admin : 1;
	unsigned int is_prot : 1;
	unsigned int is_svsmin : 1;
	unsigned int is_bot : 1;
	
	
	
};

struct stats_ {
	Stats *next;
	char name[MAXHOST];
	int numsplits;
	long maxusers;
	time_t t_maxusers;
	int maxopers;
	time_t t_maxopers;
	time_t lastseen;
	time_t starttime;
	unsigned int operkills;
	unsigned int serverkills;
};

struct ping {
    time_t last_sent;
    int ulag;
} ping;

struct daily {
	int servers;
	time_t t_servers;
	int users;
	time_t t_users;
	int opers;
	time_t t_opers;
} daily;

struct tld_ {
	TLD *next;
	char tld[4];
	char *country;
	int users;
};

struct myuser_ {
	MyUser *next;
	char *username;
	char *password;
	time_t lastseen;
	unsigned int ison : 1;
};

/* sock.c */
extern int ConnectTo(char *, int);
extern void read_loop();
extern void sts(char *, ...);
extern void log(char *, ...);
extern void ResetLogs();
extern char *sctime(time_t);
extern char *sftime(time_t);
extern void notice(char *, ...);


/* conf.c */
extern void strip(char *);
extern void ConfLoad();
extern void rehash();

/* main.c */
extern void login();
extern void init_statserv();
extern void init_spam();
extern void *smalloc(long);
extern char *sstrdup(const char *);
extern unsigned long HASH(const unsigned char *, int);
extern char *strlower(char *);

/* ircd.c */
extern void parse();
extern void privmsg(char *, const char *, char *, ...);
extern void privmsg_list(char *, char *, const char **);
extern void globops(char *, char *, ...);
extern int flood(User *);

/* timer.c */
extern void chk();
extern void TimerReset();
extern void TimerSpam();
extern void TimerPings();
extern void TimerMidnight();
extern int is_midnight();

/* statserv.c */
extern void statserv(char *);
extern void spam(char *);

/* users.c */
extern Server *serverlist[S_TABLE_SIZE];
extern User *userlist[U_TABLE_SIZE];
extern MyUser *myuhead;
extern void AddUser(char *, char *, char *, char *);
extern void DelUser(char *);
extern void Change_User(User *, char *);
extern void sendcoders(char *message);
extern User *finduser(char *);
extern void UserDump();
extern void UserMode(char *, char *);
extern void init_user_hash();
extern void AddServer(char *, int);
extern void DelServer(char *);
extern Server *findserver(char *);
extern void ServerDump();
extern void init_server_hash();
extern void LoadMyUsers();
extern void SaveMyUsers();
extern void DeleteMyUser(char *);
extern MyUser *findmyuser(char *);

/* stats.c */
extern Stats *shead;
extern TLD *tldhead;
extern char *uptime(time_t);
extern TLD *findtld(char *);
extern TLD *AddTLD(User *);
extern void LoadTLD();
extern void init_tld();
extern void IncreaseOpers(Server *);
extern void IncreaseUsers(Server *);
extern void IncreaseServers();
extern void AddStats(Server *);
extern Stats *findstats(char *);
extern void SaveStats();
extern void LoadStats();

/* help.c */
extern const char *ss_help[];
extern const char *ss_myuser_help[];
extern const char *ss_coder_help[];
extern const char *ss_server_help[];
extern const char *ss_map_help[];
extern const char *ss_netstats_help[];
extern const char *ss_daily_help[];
extern const char *ss_tld_help[];
extern const char *ss_tld_map_help[];
extern const char *ss_operlist_help[];
extern const char *ss_botlist_help[];
extern const char *ss_version_help[];
extern const char *ss_login_help[];
extern const char *ss_user_help[];
extern const char *ss_shutdown_help[];
extern const char *ss_reload_help[];
extern const char *ss_reset_help[];
extern const char *ss_stats_help[];
extern const char *ss_logs_help[];
extern const char *ss_join_help[];
extern const char *ss_raw_help[];
extern const char *ss_debug_help[];

#endif
