/* NetStats - IRC Statistical Services
** Copyright (c) 1999 Adam Rutter, Justin Hammond
** http://codeworks.kamserve.com
*
** Based from GeoStats 1.1.0 by Johnathan George net@lite.net
*
** NetStats CVS Identification
** $Id$
*/

#include "stats.h"
#include <fnmatch.h>

static Server *new_server(char *);
Server *serverlist[S_TABLE_SIZE];

MyUser *myuhead;

static User *new_user(char *);
static void add_user_to_hash_table(char *, User *);
static void del_user_from_hash_table(char *, User *);
static void add_server_to_hash_table(char *, Server *);
static void del_server_from_hash_table(char *, Server *);

User *userlist[U_TABLE_SIZE];

static void add_user_to_hash_table(char *nick, User *u)
{
	u->hash = HASH(nick, U_TABLE_SIZE);
	u->next = userlist[u->hash];
	userlist[u->hash] = (void *)u;
}

static void del_user_from_hash_table(char *nick, User *u)
{
	User *tmp, *prev = NULL;

	for (tmp = userlist[u->hash]; tmp; tmp = tmp->next) {
		if (tmp == u) {
			if (prev)
				prev->next = tmp->next;
			else
				userlist[u->hash] = tmp->next;
			tmp->next = NULL;
			return;
		}
		prev = tmp;
	}
}

static void add_server_to_hash_table(char *name, Server *s)
{
	s->hash = HASH(name, S_TABLE_SIZE);
	s->next = serverlist[s->hash];
	serverlist[s->hash] = (void *)s;
}

static void del_server_from_hash_table(char *name, Server *s)
{
	Server *tmp, *prev = NULL;

	for (tmp = serverlist[s->hash]; tmp; tmp = tmp->next) {
		if (tmp == s) {
			if (prev)
				prev->next = tmp->next;
			else
				serverlist[s->hash] = tmp->next;
			tmp->next = NULL;
			return;
		}
		prev = tmp;
	}
}

static User *new_user(char *nick)
{
	User *u;

	u = smalloc(sizeof(User));
	if (!nick)
		nick = "";
	memcpy(u->nick, nick, MAXNICK);
	add_user_to_hash_table(nick, u);

	return u;
}

void AddUser(char *nick, char *user, char *host, char *server)
{
	User *u;

#ifdef DEBUG
	log("AddUser(): %s (%s@%s) -> %s", nick, user, host, server);
#endif
	u = finduser(nick);
	if (u) {
		log("trying to add a user that already exists? (%s)", nick);
		return;
	}

	u = new_user(nick);
	u->hostname = sstrdup(host);
	u->username = sstrdup(user);
	u->server = findserver(server);
	u->is_oper = 0;
	u->t_flood = time(NULL);
	u->flood = 0;
	u->is_away = 0;
	u->is_netmin = 0;
	u->is_serv=0;
	u->is_coder=0;
	u->is_prot=0;
	u->is_tecmin = 0;
	u->is_admin=0;
	u->is_svsmin = 0;
	u->is_comin=0;
	u->is_bot =0;
	u->is_invis =0;
	u->myuser = NULL;
	u->tld = AddTLD(u);
	if (!u->server)
		log("Unable to find server for user %s (%s?)", nick, server);
	else
		IncreaseUsers(u->server);
}

void DelUser(char *nick)
{
	User *u = finduser(nick);

#ifdef DEBUG
	log("DelUser(%s)", nick);
#endif

	if (!u) {
		log("DelUser(%s) failed!", nick);
		return;
	}

	del_user_from_hash_table(nick, u);

	DecreaseUsers(u->server);

	if (u->tld)
		u->tld->users--;
	if (u->is_away)
		me.away--;
	if (u->is_oper) {
		me.opers--;
		u->server->opers--;
	}
	if (u->myuser)
		u->myuser->ison = 0;

	free(u->hostname);
	free(u->username);
	free(u);
}

void Change_User(User *u, char *newnick)
{
#ifdef DEBUG
	log("Change_User(%s, %s)", u->nick, newnick);
#endif

	del_user_from_hash_table(u->nick, u);

	u->nick[1] = 0;
	memcpy(u->nick, newnick, MAXNICK);

	add_user_to_hash_table(u->nick, u);
}
void sendcoders(char *message)
{
	User *u;
        int i;

	if (!me.coder_debug) 
		return;
		
#ifdef DEBUG
	log("FindCoders Called!");
#endif	

        for (i = 0; i < U_TABLE_SIZE; i++) {
        	for (u = userlist[i]; u; u = u->next)
			if (u->is_coder)	
			     privmsg(u->nick,s_Debug,"\2DEBUG MSG:\2 %s",message);
	}

	
}

User *finduser(char *nick)
{
	User *u;

#ifdef DEBUG
	log("finduser(%s)", nick);
#endif

	u = userlist[HASH(nick, U_TABLE_SIZE)];
	while (u && strcasecmp(u->nick, nick) != 0)
		u = u->next;

#ifdef DEBUG
	log("finduser(%s) -> %s", nick, (u) ? u->nick : "NOTFOUND");
#endif

	return u;
}

void init_user_hash()
{
	int i;
	User *u, *prev;

	for (i = 0; i < U_TABLE_SIZE; i++) {
		u = userlist[i];
		while (u) {
			prev = u->next;
#ifdef DEBUG
	log("init_user_hash() deleting %s", u->nick);
#endif
			free(u->hostname);
			free(u->username);
			free(u);

			u = prev;
		}
		userlist[i] = NULL;
	}
	bzero((char *)userlist, sizeof(userlist));
}

void UserDump()
{
#ifdef DEBUG
	User *u;
	int i;
	for (i = 0; i < U_TABLE_SIZE; i++) {
		for (u = userlist[i]; u; u = u->next)
			privmsg("netgod", s_StatServ, "User: %s", u->nick);
	}
#endif
}

void UserMode(char *nick, char *modes)
{
	User *u = finduser(nick);
	int add = 0;

	if (!u) {
		log("Changing modes for unknown user: %s", nick);
		return;
	}
	while (*modes++) {
#ifdef DEBUG
	log("Modes for %s are: %s",nick,modes);
#endif

		switch(*modes) {
			case '+': add = 1;	break;
			case '-': add = 0;	break;
			case 'N':
				if (add) {
					u->is_netmin = 1;
					notice("\2NETADMIN:\2 \2%s\2 Is Now a IRC Network Administrator (+N)",nick);
				} else {
					u->is_netmin = 0;
					notice("\2NETADMIN:\2 \2%s\2 Is no longer a IRC Network Administrator (-N)",nick);
				}
				break;
			case 'S':
				if (add) {
					u->is_serv = 1;
					notice("\2SERVICES:\2 \2%s\2 Is Marked as Services (+S)",nick);
				} else {
					u->is_serv = 0;
					notice("\2SERVICES:\2 \2%s\2 Is no longer Services (-S)",nick);
				}
				break;
			case '1':
				if (add) {
					u->is_coder = 1;
					notice("\2CODER:\2 \2%s\2 Is Marked as IRC Coder (+1)",nick);
				} else {
					u->is_coder = 0;
					notice("\2CODER:\2 \2%s\2 Is no longer a IRC Coder (-1)",nick);
				}
				break;
			case 'q':
				if (add) {
					u->is_prot = 1;
					globops(me.name,"\2%s\2 Has been Marked As Protected (+q)",nick);
					notice("\2PROTECTED:\2 \2%s\2 Is Marked as a Protected Oper (+q)",nick);
				} else {
					u->is_prot = 0;
					globops(me.name,"\2%s\2 Has been Un-Marked As Protected (-q)",nick);
					notice("\2PROTECTED:\2 \2%s\2 Is no longer a Protected Oper (-q)",nick);
				}
				break;
			case 'T':
				if (add) {
					u->is_tecmin = 1;
					notice("\2TECHADMIN:\2 \2%s\2 Is Now a IRC Technical Administrator (+T)",nick);
				} else {
					u->is_tecmin = 0;
					notice("\2TECHADMIN:\2 \2%s\2 Is no longer a IRC Technical Administrator (-T)",nick);
				}
				break;
			case 'A':
				if (add) {
					u->is_admin = 1;
					notice("\2SRVADMIN:\2 \2%s\2 Is Now a Server Administrator (+A)",nick);
				} else {
					u->is_admin = 0;
					notice("\2SRVADMIN:\2 \2%s\2 Is no longer a Server Administrator (-A)",nick);
				}
				break;
			case 'a':
				if (add) {
					u->is_svsmin = 1;
					notice("\2SERVICESADMIN:\2 \2%s\2 Is Now a Services Administrator (+a)",nick);
				} else {
					u->is_svsmin = 0;
					notice("\2SERVICESADMIN:\2 \2%s\2 Is no longer a Services Administrator (-a)",nick);
				}
				break;
			case 'C':
				if (add) {
					u->is_comin = 1;
					notice("\2CO-SRVADMIN:\2 \2%s\2 Is Now a Co-Server Administrator (+C)",nick);
				} else {
					u->is_comin = 0;
					notice("\2CO-SRVADMIN:\2 \2%s\2 Is no longer a Services Administrator (-C)",nick);
				}
				break;
			case 'B':
				if (add) {
					u->is_bot = 1;
					notice("\2BOT:\2 \2%s\2 Is Now a BOT (+B)",nick);
				} else {
					u->is_bot = 0;
					notice("\2BOT:\2 \2%s\2 Is no longer a BOT (-B)",nick);
				}
				break;
			case 'I':
				if (add) {
					u->is_invis = 1;
					globops(me.name,"\2%s\2 Is Using \2Invisible Mode\2 (+I)",nick);
					notice("\2INVISIBLE:\2 \2%s\2 is Using Invisible Mode (+I)",nick);
				} else {
					u->is_invis = 0;
					globops(me.name,"\2%s\2 Is no longer using \2Invisible Mode\2 (-I)",nick);
					notice("\2INVISIBLE:\2 \2%s\2 is no longer Using Invisible Mode (+I)",nick);
				}
				break;
			case 'o':
				if (add) {
					u->is_oper = 1;
					IncreaseOpers(u->server);
					notice("\2OPER:\2 \2%s\2 Is Now a IRCOP! (+o)",nick);
#ifdef DEBUG
					log("%s is now an IRCop.", u->nick);
#endif
				} else {
					u->is_oper = 0;
					me.opers--;
					u->server->opers--;
					notice("\2OPER:\2 \2%s\2 Is no longer a IRCOP! (+o)",nick);

#ifdef DEBUG
					log("%s is no longer an IRCop.", u->nick);
#endif
				}
				break;
			case 'O':
				if (add) {
					u->is_oper = 1;
					IncreaseOpers(u->server);
					notice("\2OPER:\2 \2%s\2 Is Now a Local IRCOP! (+o)",nick);
#ifdef DEBUG
					log("%s is now an IRCop.", u->nick);
#endif
				} else {
					u->is_oper = 0;
					me.opers--;
					u->server->opers--;
					notice("\2OPER:\2 \2%s\2 Is no longer a Local IRC OP! (+o)",nick);

#ifdef DEBUG
					log("%s is no longer an IRCop.", u->nick);
#endif
				}
				break;
			default:
				break;
		}
	}
}

static Server *new_server(char *name)
{
	Server *s;

	s = calloc(sizeof(Server), 1);
	if (!name)
		name = "";
	memcpy(s->name, name, MAXHOST);
	add_server_to_hash_table(name, s);

	return s;
}

void AddServer(char *name, int hops)
{
	Server *s;

#ifdef DEBUG
	log("New Server: %s", name);
#endif
	strlower(name);
	s = new_server(name);
	s->hops = hops;
	s->connected_since = time(NULL);
	s->last_announce = time(NULL);
	s->ping = -1;
	s->lowest_ping = 0;
	s->highest_ping = 0;
	s->t_lowest_ping = time(NULL);
	s->t_highest_ping = time(NULL);
	IncreaseServers();
	AddStats(s);
}

void DelServer(char *name)
{
	Server *s = findserver(name);

	if (!s || !name) {
		log("DelServer(): %s failed!", name);
		return;
	}

	del_server_from_hash_table(name, s);
	s->stats->lastseen = time(NULL);
	DecreaseServers();
	free(s);
}

Server *findserver(char *name)
{
	Server *s;

	strlower(name);
#ifdef DEBUG
	log("findserver(%s)", name);
#endif
	s = serverlist[HASH(name, S_TABLE_SIZE)];
	while (s && strcmp(s->name, name) != 0)
		s = s->next;
	if (s)
		return s;

	s = serverlist[HASH(name, S_TABLE_SIZE)];
	while (s && fnmatch(name, s->name, 0) != 0)
		s = s->next;

#ifdef DEBUG
	log("findserver(%s) -> %s", name, (s) ? s->name : "NOT FOUND");
#endif

	return s;
}

void ServerDump()
{
#ifdef DEBUG
	Server *s;
	register int j;

	privmsg("netgod", s_StatServ, "Server Listing:");
	for (j = 0; j < S_TABLE_SIZE; j++) {
		for (s = serverlist[j]; s; s = s->next) {
			privmsg("netgod", s_StatServ, "Server Entry: %s [%d/%d] %d", 
				s->name, s->users, s->stats->maxusers, s->opers);
			privmsg("netgod", s_StatServ, "    Stats: %d / %d / %s",
				s->stats->maxusers, s->stats->maxopers,
				sctime(s->connected_since));
		}
	}
	privmsg("netgod", s_StatServ, "End of Listing.");
#endif
}

void init_server_hash()
{
	int i;
	Server *s, *b;

	for (i = 0; i < S_TABLE_SIZE; i++) {
		s = serverlist[i];
		while (s) {
			b = s->next;
#ifdef DEBUG
	log("init_server_hash() deleting %s", s->name);
#endif
			free(s);
			s = b;
		}
		serverlist[i] = NULL;
	}
	bzero((char *)serverlist, sizeof(serverlist));
	AddServer(me.name, 0);
}

